@@ -1,15 +1,21 @@
 /*
-   Copyright (C) 2003-2008 MySQL AB
-    All rights reserved. Use is subject to license terms.
+   Copyright (c) 2003, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -29,8 +35,9 @@
 struct TriggerType {
   enum Value {
     //CONSTRAINT            = 0,
-    SECONDARY_INDEX       = DictTabInfo::HashIndexTrigger,
-    //FOREIGN_KEY           = 2,
+    SECONDARY_INDEX         = DictTabInfo::HashIndexTrigger,
+    FK_PARENT               = DictTabInfo::FKParentTrigger,
+    FK_CHILD                = DictTabInfo::FKChildTrigger,
     //SCHEMA_UPGRADE        = 3,
     //API_TRIGGER           = 4,
     //SQL_TRIGGER           = 5,
@@ -153,6 +160,10 @@ struct TriggerInfo {
     switch (val) {
     case TriggerType::SECONDARY_INDEX:
       return "SECONDARY_INDEX";
+    case TriggerType::FK_PARENT:
+      return "FK_PARENT";
+    case TriggerType::FK_CHILD:
+      return "FK_CHILD";
     case TriggerType::SUBSCRIPTION:
       return "SUBSCRIPTION";
     case TriggerType::READ_ONLY_CONSTRAINT:
@@ -161,6 +172,8 @@ struct TriggerInfo {
       return "ORDERED_INDEX";
     case TriggerType::SUBSCRIPTION_BEFORE:
       return "SUBSCRIPTION_BEFORE";
+    case TriggerType::REORG_TRIGGER:
+      return "REORG_TRIGGER";
     }
     return "UNKNOWN";
   }
@@ -199,7 +212,8 @@ struct TriggerInfo {
 struct NoOfFiredTriggers
 {
   STATIC_CONST( DeferredUKBit = (Uint32(1) << 31) );
-  STATIC_CONST( DeferredBits = (DeferredUKBit));
+  STATIC_CONST( DeferredFKBit = (Uint32(1) << 30) );
+  STATIC_CONST( DeferredBits = (DeferredUKBit | DeferredFKBit));
 
   static Uint32 getFiredCount(Uint32 v) {
     return v & ~(Uint32(DeferredBits));
@@ -207,9 +221,40 @@ struct NoOfFiredTriggers
   static Uint32 getDeferredUKBit(Uint32 v) {
     return (v & Uint32(DeferredUKBit)) != 0;
   }
-  static void setDeferredBit(Uint32 & v) {
+  static void setDeferredUKBit(Uint32 & v) {
     v |= Uint32(DeferredUKBit);
   }
+  static Uint32 getDeferredFKBit(Uint32 v) {
+    return (v & Uint32(DeferredFKBit)) != 0;
+  }
+  static void setDeferredFKBit(Uint32 & v) {
+    v |= Uint32(DeferredFKBit);
+  }
+
+  static bool getDeferredAllSet(Uint32 v) {
+    return (v & Uint32(DeferredBits)) == DeferredBits;
+  }
+};
+
+struct TriggerPreCommitPass
+{
+  /**
+   * When using deferred triggers...
+   * - UK are split into 2 passes...
+   * - FK needs to be evaluated *after* UK has been processed
+   *   as it (can) use UK
+   *
+   * When having cascadeing FK's they can provoke UK updates
+   *   in such cases...the passes are
+   *   N * (PASS_MAX + 1) + PASS
+   */
+  enum
+  {
+    UK_PASS_0 = 0,
+    UK_PASS_1 = 1,
+    FK_PASS_0 = 7, // leave some room...(unsure if it's needed)
+    TPCP_PASS_MAX = 15
+  };
 };
 
 #endif
