@@ -1,14 +1,21 @@
 /*
-   Copyright (c) 2003, 2011, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; version 2 of the License.
+   it under the terms of the GNU General Public License, version 2.0,
+   as published by the Free Software Foundation.
+
+   This program is also distributed with certain software (including
+   but not limited to OpenSSL) that is licensed under separate terms,
+   as designated in a particular file or component or in included license
+   documentation.  The authors of MySQL hereby grant you an additional
+   permission to link the program and your derivative works with the
+   separately licensed software that they have included with MySQL.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+   GNU General Public License, version 2.0, for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -22,6 +29,9 @@
 #include <ndb_limits.h>
 #include "DblqhCommon.hpp"
 
+#define JAM_FILE_ID 452
+
+
 #define DEBUG(x) { ndbout << "LQH::" << x << endl; }
 
 void Dblqh::initData() 
@@ -64,17 +74,25 @@ void Dblqh::initData()
 
   cLqhTimeOutCount = 0;
   cLqhTimeOutCheckCount = 0;
-  cbookedAccOps = 0;
   cpackedListIndex = 0;
   m_backup_ptr = RNIL;
   clogFileSize = 16;
   cmaxLogFilesInPageZero = 40;
+  cmaxValidLogFilesInPageZero = cmaxLogFilesInPageZero - 1;
 
-   totalLogFiles = 0;
-   logFileInitDone = 0;
-   totallogMBytes = 0;
-   logMBytesInitDone = 0;
-   m_startup_report_frequency = 0;
+#if defined VM_TRACE || defined ERROR_INSERT
+  cmaxLogFilesInPageZero_DUMP = 0;
+#endif
+
+#if defined ERROR_INSERT
+  delayOpenFilePtrI = 0;
+#endif
+
+  totalLogFiles = 0;
+  logFileInitDone = 0;
+  totallogMBytes = 0;
+  logMBytesInitDone = 0;
+  m_startup_report_frequency = 0;
 
   c_active_add_frag_ptr_i = RNIL;
   for (Uint32 i = 0; i < 1024; i++) {
@@ -91,8 +109,32 @@ void Dblqh::initData()
 
   c_max_parallel_scans_per_frag = 32;
 
+  c_lcpFragWatchdog.block = this;
   c_lcpFragWatchdog.reset();
   c_lcpFragWatchdog.thread_active = false;
+
+  c_keyOverloads           = 0;
+  c_keyOverloadsTcNode     = 0;
+  c_keyOverloadsReaderApi  = 0;
+  c_keyOverloadsPeerNode   = 0;
+  c_keyOverloadsSubscriber = 0;
+  c_scanSlowDowns          = 0;
+
+  c_fragmentsStarted = 0;
+  c_fragmentsStartedWithCopy = 0;
+
+  c_fragCopyTable = 0;
+  c_fragCopyFrag = 0;
+  c_fragCopyRowsIns = 0;
+  c_fragCopyRowsDel = 0;
+  c_fragBytesCopied = 0;
+
+  c_fragmentCopyStart = 0;
+  c_fragmentsCopied = 0;
+  c_totalCopyRowsIns = 0;
+  c_totalCopyRowsDel = 0;
+  c_totalBytesCopied = 0;
+
 }//Dblqh::initData()
 
 void Dblqh::initRecords() 
@@ -267,6 +309,7 @@ Dblqh::Dblqh(Block_context& ctx, Uint32 instanceNumber):
   c_lcp_waiting_fragments(c_fragment_pool),
   c_lcp_restoring_fragments(c_fragment_pool),
   c_lcp_complete_fragments(c_fragment_pool),
+  c_queued_lcp_frag_ord(c_fragment_pool),
   m_commitAckMarkerHash(m_commitAckMarkerPool),
   c_scanTakeOverHash(c_scanRecordPool)
 {
@@ -335,7 +378,6 @@ Dblqh::Dblqh(Block_context& ctx, Uint32 instanceNumber):
   addRecSignal(GSN_TUPSEIZEREF, &Dblqh::execTUPSEIZEREF);
   addRecSignal(GSN_ACCKEYCONF, &Dblqh::execACCKEYCONF);
   addRecSignal(GSN_ACCKEYREF, &Dblqh::execACCKEYREF);
-  addRecSignal(GSN_TUPKEYCONF, &Dblqh::execTUPKEYCONF);
   addRecSignal(GSN_TUPKEYREF, &Dblqh::execTUPKEYREF);
   addRecSignal(GSN_ABORT, &Dblqh::execABORT);
   addRecSignal(GSN_ABORTREQ, &Dblqh::execABORTREQ);
@@ -346,17 +388,12 @@ Dblqh::Dblqh(Block_context& ctx, Uint32 instanceNumber):
 #endif
   addRecSignal(GSN_SCAN_FRAGREQ, &Dblqh::execSCAN_FRAGREQ);
   addRecSignal(GSN_SCAN_NEXTREQ, &Dblqh::execSCAN_NEXTREQ);
-  addRecSignal(GSN_ACC_SCANCONF, &Dblqh::execACC_SCANCONF);
-  addRecSignal(GSN_ACC_SCANREF, &Dblqh::execACC_SCANREF);
   addRecSignal(GSN_NEXT_SCANCONF, &Dblqh::execNEXT_SCANCONF);
   addRecSignal(GSN_NEXT_SCANREF, &Dblqh::execNEXT_SCANREF);
-  addRecSignal(GSN_STORED_PROCCONF, &Dblqh::execSTORED_PROCCONF);
-  addRecSignal(GSN_STORED_PROCREF, &Dblqh::execSTORED_PROCREF);
   addRecSignal(GSN_COPY_FRAGREQ, &Dblqh::execCOPY_FRAGREQ);
   addRecSignal(GSN_COPY_FRAGREF, &Dblqh::execCOPY_FRAGREF);
   addRecSignal(GSN_COPY_FRAGCONF, &Dblqh::execCOPY_FRAGCONF);
   addRecSignal(GSN_COPY_ACTIVEREQ, &Dblqh::execCOPY_ACTIVEREQ);
-  addRecSignal(GSN_COPY_STATEREQ, &Dblqh::execCOPY_STATEREQ);
   addRecSignal(GSN_LQH_TRANSREQ, &Dblqh::execLQH_TRANSREQ);
   addRecSignal(GSN_TRANSID_AI, &Dblqh::execTRANSID_AI);
   addRecSignal(GSN_INCL_NODEREQ, &Dblqh::execINCL_NODEREQ);
@@ -391,7 +428,6 @@ Dblqh::Dblqh(Block_context& ctx, Uint32 instanceNumber):
   addRecSignal(GSN_DROP_TAB_REF, &Dblqh::execDROP_TAB_REF);
   addRecSignal(GSN_DROP_TAB_CONF, &Dblqh::execDROP_TAB_CONF);
 
-  addRecSignal(GSN_LQH_ALLOCREQ, &Dblqh::execLQH_ALLOCREQ);
   addRecSignal(GSN_LQH_WRITELOG_REQ, &Dblqh::execLQH_WRITELOG_REQ);
   addRecSignal(GSN_TUP_DEALLOCREQ, &Dblqh::execTUP_DEALLOCREQ);
 
@@ -449,7 +485,6 @@ Dblqh::Dblqh(Block_context& ctx, Uint32 instanceNumber):
       &scanptr,
       &tabptr,
       &tcConnectptr,
-      &tcNodeFailptr,
     }; 
     init_globals_list(tmp, sizeof(tmp)/sizeof(tmp[0]));
   }
