@@ -1,14 +1,21 @@
 /*
-  Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2011, 2015, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; version 2 of the License.
+  it under the terms of the GNU General Public License, version 2.0,
+  as published by the Free Software Foundation.
+
+  This program is also distributed with certain software (including
+  but not limited to OpenSSL) that is licensed under separate terms,
+  as designated in a particular file or component or in included license
+  documentation.  The authors of MySQL hereby grant you an additional
+  permission to link the program and your derivative works with the
+  separately licensed software that they have included with MySQL.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
+  GNU General Public License, version 2.0, for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -24,6 +31,11 @@
 #include <signaldata/DumpStateOrd.hpp>
 
 #include <mt.hpp>
+#include <EventLogger.hpp>
+extern EventLogger * g_eventLogger;
+
+#define JAM_FILE_ID 430
+
 
 Trpman::Trpman(Block_context & ctx, Uint32 instanceno) :
   SimulatedBlock(TRPMAN, ctx, instanceno)
@@ -57,13 +69,12 @@ extern Uint32 MAX_RECEIVED_SIGNALS;
 bool
 Trpman::handles_this_node(Uint32 nodeId)
 {
-#ifndef NDBD_MULTITHREADED
-  return true;
-#else
+  /* If there's only one receiver then no question */
   if (globalData.ndbMtReceiveThreads <= (Uint32)1)
     return true;
-  return (instance()== (mt_get_recv_thread_idx(nodeId) + /* proxy */ 1));
-#endif
+  
+  /* There's a global receiver->thread index - look it up */
+  return (instance() == (get_recv_thread_idx(nodeId) + /* proxy */ 1));
 }
 
 void
@@ -184,27 +195,32 @@ Trpman::execCLOSE_COMREQ(Signal* signal)
   const BlockReference userRef = closeCom->xxxBlockRef;
   Uint32 requestType = closeCom->requestType;
   Uint32 failNo = closeCom->failNo;
-//  Uint32 noOfNodes = closeCom->noOfNodes;
+  Uint32 noOfNodes = closeCom->noOfNodes;
+  Uint32 found_nodes = 0;
 
   jamEntry();
   for (unsigned i = 1; i < MAX_NODES; i++)
   {
-    if (NodeBitmask::get(closeCom->theNodes, i) &&
-        handles_this_node(i))
+    if (NodeBitmask::get(closeCom->theNodes, i))
     {
-      jam();
+      found_nodes++;
+      if (handles_this_node(i))
+      {
+        jam();
 
-      //-----------------------------------------------------
-      // Report that the connection to the node is closed
-      //-----------------------------------------------------
-      signal->theData[0] = NDB_LE_CommunicationClosed;
-      signal->theData[1] = i;
-      sendSignal(CMVMI_REF, GSN_EVENT_REP, signal, 2, JBB);
+        //-----------------------------------------------------
+        // Report that the connection to the node is closed
+        //-----------------------------------------------------
+        signal->theData[0] = NDB_LE_CommunicationClosed;
+        signal->theData[1] = i;
+        sendSignal(CMVMI_REF, GSN_EVENT_REP, signal, 2, JBB);
 
-      globalTransporterRegistry.setIOState(i, HaltIO);
-      globalTransporterRegistry.do_disconnect(i);
+        globalTransporterRegistry.setIOState(i, HaltIO);
+        globalTransporterRegistry.do_disconnect(i);
+      }
     }
   }
+  ndbrequire(noOfNodes == found_nodes);
 
   if (requestType != CloseComReqConf::RT_NO_REPLY)
   {
@@ -375,6 +391,7 @@ Trpman::execDBINFO_SCANREQ(Signal *signal)
   const Ndbinfo::ScanCursor* cursor =
     CAST_CONSTPTR(Ndbinfo::ScanCursor, DbinfoScan::getCursorPtr(&req));
   Ndbinfo::Ratelimit rl;
+  char addr_buf[NDB_ADDR_STRLEN];
 
   jamEntry();
 
@@ -404,27 +421,50 @@ Trpman::execDBINFO_SCANREQ(Signal *signal)
         row.write_uint32(rnode); // Remote node id
         row.write_uint32(globalTransporterRegistry.getPerformState(rnode)); // State
 
-        /* Connect address */
-        if (globalTransporterRegistry.get_transporter(rnode) != NULL &&
-            globalTransporterRegistry.get_connect_address(rnode).s_addr != 0)
-        {
-          row.write_string(inet_ntoa(globalTransporterRegistry.get_connect_address(rnode)));
-        }
-        else
-        {
-          row.write_string("-");
-        }
-
-        /* Bytes sent/received */
         if (globalTransporterRegistry.get_transporter(rnode) != NULL)
         {
+          jam();
+          /* Connect address */
+          if (globalTransporterRegistry.get_connect_address(rnode).s_addr != 0)
+          {
+            jam();
+            struct in_addr conn_addr = globalTransporterRegistry.
+                                         get_connect_address(rnode);
+            char *addr_str = Ndb_inet_ntop(AF_INET,
+                                           static_cast<void*>(&conn_addr),
+                                           addr_buf,
+                                           (socklen_t)sizeof(addr_buf));
+            row.write_string(addr_str);
+          }
+          else
+          {
+            jam();
+            row.write_string("-");
+          }
+          
+          /* Bytes sent/received */
           row.write_uint64(globalTransporterRegistry.get_bytes_sent(rnode));
           row.write_uint64(globalTransporterRegistry.get_bytes_received(rnode));
+          
+          /* Connect count, overload and Slowdown states */
+          row.write_uint32(globalTransporterRegistry.get_connect_count(rnode));
+          row.write_uint32(globalTransporterRegistry.get_status_overloaded().get(rnode));
+          row.write_uint32(globalTransporterRegistry.get_overload_count(rnode));
+          row.write_uint32(globalTransporterRegistry.get_status_slowdown().get(rnode));
+          row.write_uint32(globalTransporterRegistry.get_slowdown_count(rnode));
         }
         else
         {
-          row.write_uint64(0);
-          row.write_uint64(0);
+          /* Null transporter */
+          jam();
+          row.write_string("-");  /* Remote address */
+          row.write_uint64(0);    /* Bytes sent */
+          row.write_uint64(0);    /* Bytes received */
+          row.write_uint32(0);    /* Connect count */
+          row.write_uint32(0);    /* Overloaded */
+          row.write_uint32(0);    /* Overload_count */
+          row.write_uint32(0);    /* Slowdown */
+          row.write_uint32(0);    /* Slowdown_count */
         }
 
         ndbinfo_send_row(signal, req, row, rl);
@@ -562,12 +602,13 @@ Trpman::execDUMP_STATE_ORD(Signal* signal)
       {
         if (block)
         {
-          ndbout_c("TRPMAN : Blocking receive from node %u", nodeId);
+          g_eventLogger->info("TRPMAN : Blocking receive from node %u", nodeId);
           globalTransporterRegistry.blockReceive(*recvdata, nodeId);
         }
         else
         {
-          ndbout_c("TRPMAN : Unblocking receive from node %u", nodeId);
+          g_eventLogger->info("TRPMAN : Unblocking receive from node %u", 
+                              nodeId);
 
           globalTransporterRegistry.unblockReceive(*recvdata, nodeId);
         }
@@ -619,7 +660,7 @@ Trpman::execDUMP_STATE_ORD(Signal* signal)
             default:
               break;
             }
-            ndbout_c("TRPMAN : Blocking receive from node %u", node);
+            g_eventLogger->info("TRPMAN : Blocking receive from node %u", node);
             globalTransporterRegistry.blockReceive(*recvdata, node);
           }
         }
@@ -636,11 +677,52 @@ Trpman::execDUMP_STATE_ORD(Signal* signal)
         continue;
       if (globalTransporterRegistry.isBlocked(node))
       {
-        ndbout_c("CMVMI : Unblocking receive from node %u", node);
+        g_eventLogger->info("TRPMAN : Unblocking receive from node %u", node);
         globalTransporterRegistry.unblockReceive(*recvdata, node);
       }
     }
   }
+  if (arg == 9994 ||  /* Block send to node X */
+      arg == 9995)    /* Unblock send to node X */
+  {
+    bool block = (arg == 9994);
+    TransporterReceiveHandle * recvdata = mt_get_trp_receive_handle(instance());
+    assert(recvdata != 0);
+    for (Uint32 n = 1; n < signal->getLength(); n++)
+    {
+      Uint32 nodeId = signal->theData[n];
+      if (!handles_this_node(nodeId))
+        continue;
+
+      if ((nodeId > 0) &&
+          (nodeId < MAX_NODES))
+      {
+        g_eventLogger->info("TRPMAN : Send to %u is %sblocked",
+                            nodeId, 
+                            (globalTransporterRegistry.
+                             isSendBlocked(nodeId)?"":"not "));
+        if (block)
+        {
+          g_eventLogger->info("TRPMAN : Blocking send to node %u", nodeId);
+          globalTransporterRegistry.blockSend(*recvdata, nodeId);
+        }
+        else
+        {
+          g_eventLogger->info("TRPMAN : Unblocking send to node %u", 
+                              nodeId);
+
+          globalTransporterRegistry.unblockSend(*recvdata, nodeId);
+        }
+      }
+      else
+      {
+        ndbout_c("TRPMAN : Ignoring dump %u for node %u",
+                 arg, nodeId);
+      }
+    }
+
+  }
+
 #endif
 }
 
@@ -797,17 +879,21 @@ TrpmanProxy::sendENABLE_COMCONF(Signal *signal, Uint32 ssId)
 void
 TrpmanProxy::execROUTE_ORD(Signal* signal)
 {
-  RouteOrd* ord = (RouteOrd*)signal->getDataPtr();
-  Uint32 nodeId = ord->from;
   jamEntry();
 
+  RouteOrd* ord = (RouteOrd*)signal->getDataPtr();
+  Uint32 nodeId = ord->from;
   ndbassert(nodeId != 0);
-#ifndef NDBD_MULTITHREADED
-  Uint32 workerId = 0;
-#else
-  Uint32 workerId = mt_get_recv_thread_idx(nodeId);
-#endif
+
+  Uint32 workerIndex = 0;
+  
+  if (globalData.ndbMtReceiveThreads > (Uint32) 1)
+  {
+    workerIndex = get_recv_thread_idx(nodeId);
+    ndbrequire(workerIndex < globalData.ndbMtReceiveThreads);
+  }
+  
   SectionHandle handle(this, signal);
-  sendSignal(workerRef(workerId), GSN_ROUTE_ORD, signal,
+  sendSignal(workerRef(workerIndex), GSN_ROUTE_ORD, signal,
              signal->getLength(), JBB, &handle);
 }
