@@ -39,6 +39,7 @@ Name:          Ndb.cpp
 #include <NdbEnv.h>
 #include <BaseString.hpp>
 #include <NdbSqlUtil.hpp>
+#include <NdbTick.h>
 
 /****************************************************************************
 void connect();
@@ -939,16 +940,27 @@ NdbTransaction*
 Ndb::startTransactionLocal(Uint32 aPriority, Uint32 nodeId, Uint32 instance)
 {
 #ifdef VM_TRACE
+#ifdef NDB_USE_GET_ENV
   char buf[255];
   const char* val = NdbEnv_GetEnv("NDB_TRANSACTION_NODE_ID", buf, 255);
   if(val != 0){
     nodeId = atoi(val);
   }
+#endif
 #endif
 
   DBUG_ENTER("Ndb::startTransactionLocal");
   DBUG_PRINT("enter", ("nodeid: %d", nodeId));
 
+#ifdef VM_TRACE
+  DBUG_EXECUTE_IF("ndb_start_transaction_fail",
+                  {
+                    /* Cluster failure */
+                    theError.code = 4009;
+                    DBUG_RETURN(0);
+                  };);
+#endif
+
   if(unlikely(theRemainingStartTransactions == 0))
   {
     theError.code = 4006;
@@ -1892,9 +1904,15 @@ Ndb::externalizeTableName(const char * internalTableName, bool fullyQualifiedNam
     register const char *ptr = internalTableName;
    
     // Skip database name
-    while (*ptr && *ptr++ != table_name_separator);
+    while (*ptr && *ptr++ != table_name_separator)
+    {
+      ;
+    }
     // Skip schema name
-    while (*ptr && *ptr++ != table_name_separator);
+    while (*ptr && *ptr++ != table_name_separator)
+    {
+      ;
+    }
     return ptr;
   }
   else
@@ -1914,14 +1932,23 @@ Ndb::externalizeIndexName(const char * internalIndexName, bool fullyQualifiedNam
     register const char *ptr = internalIndexName;
    
     // Scan name from the end
-    while (*ptr++); ptr--; // strend
+    while (*ptr++)
+    {
+      ;
+    }
+    ptr--; // strend
+
     while (ptr >= internalIndexName && *ptr != table_name_separator)
+    {
       ptr--;
+    }
      
     return ptr + 1;
   }
   else
+  {
     return internalIndexName;
+  }
 }
 
 const char *
@@ -2088,6 +2115,28 @@ Ndb::set_eventbuf_max_alloc(unsigned sz)
   }
 }
 
+unsigned Ndb::get_eventbuffer_free_percent()
+{
+  return theEventBuffer->get_eventbuffer_free_percent();
+}
+
+int
+Ndb::set_eventbuffer_free_percent(unsigned free)
+{
+  if (free < 1 || free > 99)
+  {
+    theError.code = 4123;
+    return -1;
+  }
+  theEventBuffer->set_eventbuffer_free_percent(free);
+  return 0;
+}
+
+void Ndb::get_event_buffer_memory_usage(EventBufferMemoryUsage& usage)
+{
+  theEventBuffer->get_event_buffer_memory_usage(usage);
+}
+
 NdbEventOperation* Ndb::createEventOperation(const char* eventName)
 {
   DBUG_ENTER("Ndb::createEventOperation");
@@ -2096,6 +2145,8 @@ NdbEventOperation* Ndb::createEventOperation(const char* eventName)
   if (tOp)
   {
     // keep track of all event operations
+    // Serialize changes to m_ev_op with dropEventOperation
+    theImpl->lock();
     NdbEventOperationImpl *op=
       NdbEventBuffer::getEventOperationImpl(tOp);
     op->m_next= theImpl->m_ev_op;
@@ -2103,6 +2154,7 @@ NdbEventOperation* Ndb::createEventOperation(const char* eventName)
     theImpl->m_ev_op= op;
     if (op->m_next)
       op->m_next->m_prev= op;
+    theImpl->unlock();
   }
 
   DBUG_RETURN(tOp);
@@ -2131,10 +2183,19 @@ NdbEventOperation *Ndb::getEventOperation(NdbEventOperation* tOp)
 }
 
 int
-Ndb::pollEvents(int aMillisecondNumber, Uint64 *latestGCI)
+Ndb::pollEvents2(int aMillisecondNumber, Uint64 *highestQueuedEpoch)
 {
+  if (unlikely(aMillisecondNumber < 0))
+  {
+    g_eventLogger->error("Ndb::pollEvents2: negative aMillisecondNumber %d 0x%x %s",
+                         aMillisecondNumber,
+                         getReference(),
+                         getNdbObjectName());
+    return -1;
+  }
+
   /* Look for already available events without polling transporter. */
-  const int found = theEventBuffer->pollEvents(latestGCI);
+  const int found = theEventBuffer->pollEvents(highestQueuedEpoch);
   if (found)
     return found;
 
@@ -2147,7 +2208,110 @@ Ndb::pollEvents(int aMillisecondNumber, Uint64 *latestGCI)
   poll_guard.wait_n_unlock(aMillisecondNumber, 0, WAIT_EVENT);
   // PollGuard ends here
 
-  return theEventBuffer->pollEvents(latestGCI);
+  return theEventBuffer->pollEvents(highestQueuedEpoch);
+}
+
+bool
+Ndb::isExpectingHigherQueuedEpochs()
+{
+  return !theEventBuffer->m_failure_detected;
+}
+
+void
+Ndb::printOverflowErrorAndExit()
+{
+  g_eventLogger->error("Ndb Event Buffer : 0x%x %s",
+                       getReference(), getNdbObjectName());
+  g_eventLogger->error("Ndb Event Buffer : Event buffer out of memory.");
+  g_eventLogger->error("Ndb Event Buffer : Fatal error.");
+  Uint32 maxalloc = get_eventbuf_max_alloc();
+  if (maxalloc != 0)
+  {
+    // limited memory is allocated for event buffer, give recommendation
+    g_eventLogger->error("Ndb Event Buffer : Change eventbuf_max_alloc (Current max_alloc is %u).", maxalloc);
+  }
+  g_eventLogger->error("Ndb Event Buffer : Consider using the new API.");
+  exit(-1);
+}
+
+int
+Ndb::pollEvents(int aMillisecondNumber, Uint64 *latestGCI)
+{
+  /**
+   * aMillisecondNumber = 0 : one poll call
+   * else divide aMillisecondNumber into a 1-hour time slot
+   * and call poll with this time slot until it expires.
+   * Note: The else-part: it will cover aMillisecondNumber < 0,
+   * which is converted to unsigned and become large,
+   * as well as aMillisecondNumber > 0.
+   */
+  const Uint32 waitSlot = 3600*1000; // in millisecs
+  const Uint32 totalWaitTime = (Uint32)aMillisecondNumber;
+  const NDB_TICKS startTime = NdbTick_getCurrentTicks();
+  Uint32 remaining = totalWaitTime;
+  Uint32 waited = 0;
+
+  do
+  {
+    const Uint32 pollTimeout = (remaining > waitSlot) ? waitSlot :
+      remaining;
+    const int res = pollEvents2((int)pollTimeout, latestGCI);
+
+    if ((latestGCI) && (isExpectingHigherQueuedEpochs() == false))
+      *latestGCI= NDB_FAILURE_GCI;
+
+    if (res < 0)
+    {
+      return res;
+    }
+
+    if (res > 0)
+    {
+      EventBufData *data = theEventBuffer->m_available_data.m_head;
+      while (data)
+      {
+        // All including exceptional event data must have an associated buffer
+        assert(data->sdata);
+
+        const Uint32 type =
+          SubTableData::getOperation(data->sdata->requestInfo);
+
+        if ((type != NdbDictionary::Event::_TE_EMPTY))
+        {
+          /* res >0  will be returned here, making the consumer
+           * to call nextEvent().
+           * nextEvent() call will handle the new types:
+           * _TE_INCONSISTENT and _TE_OUT_OF_MEMORY:
+           *
+           * Consumer handles other types as usual.
+           */
+          return res;
+	}
+
+        // Consumer cannot handle the new type TE_EMPTY, filter it.
+        (void)nextEvent2();
+        data = theEventBuffer->m_available_data.m_head;
+      }
+      // Event queue is scanned and no regular event data is found
+      assert(data==NULL);
+
+      /* This is intentional: Return with 'no data' even
+       * if this epoch did not contain any data-events - Even
+       * if the specified wait time didn't expire. Reason is
+       * to give the client a chance to observe, and act on,
+       * possibly new epoch which has completed.
+       */
+      return 0;
+    }
+
+    waited =
+      (Uint32)NdbTick_Elapsed(startTime, NdbTick_getCurrentTicks()).milliSec();
+
+    remaining = totalWaitTime - waited;
+
+  } while (totalWaitTime > waited);
+
+  return 0;
 }
 
 int
@@ -2159,9 +2323,35 @@ Ndb::flushIncompleteEvents(Uint64 gci)
   return ret;
 }
 
+NdbEventOperation *Ndb::nextEvent2()
+{
+  return theEventBuffer->nextEvent2();
+}
+
 NdbEventOperation *Ndb::nextEvent()
 {
-  return theEventBuffer->nextEvent();
+  NdbDictionary::Event::TableEvent errType;
+
+  // Remove the event data from the head
+  NdbEventOperation *op = nextEvent2();
+
+  while (op)
+  {
+    if (op->isErrorEpoch(&errType))
+    {
+      if (errType ==  NdbDictionary::Event::TE_INCONSISTENT)
+        return NULL;
+
+      if (errType ==  NdbDictionary::Event::TE_OUT_OF_MEMORY)
+        printOverflowErrorAndExit();
+    }
+
+    if (!op->isEmptyEpoch())
+      break; // return non-empty epoch
+
+    op = nextEvent2(); // remove empty epoch and check the next one
+  }
+  return op;
 }
 
 bool
@@ -2177,7 +2367,7 @@ Ndb::isConsistentGCI(Uint64 gci)
 }
 
 const NdbEventOperation*
-Ndb::getGCIEventOperations(Uint32* iter, Uint32* event_types)
+Ndb::getNextEventOpInEpoch2(Uint32* iter, Uint32* event_types)
 {
   return getNextEventOpInEpoch3(iter, event_types, NULL);
 }
@@ -2193,6 +2383,21 @@ Ndb::getNextEventOpInEpoch3(Uint32* iter, Uint32* event_types,
   return NULL;
 }
 
+const NdbEventOperation*
+Ndb::getGCIEventOperations(Uint32* iter, Uint32* event_types)
+{
+  return getNextEventOpInEpoch3(iter, event_types, NULL);
+  /*
+   * No event operation is added to gci_ops list for exceptional event data.
+   * So it is not possible to get them in event_types. No check needed.
+   */
+}
+
+Uint64 Ndb::getHighestQueuedEpoch()
+{
+  return theEventBuffer->getHighestQueuedEpoch();
+}
+
 Uint64 Ndb::getLatestGCI()
 {
   return theEventBuffer->getLatestGCI();
@@ -2345,8 +2550,7 @@ Ndb::getNdbErrorDetail(const NdbError& err, char* buff, Uint32 buffLen) const
             
             Uint32 components = idxName.split(idxNameComponents,
                                               splitString);
-            
-            assert(components == 4);
+            require(components == 4);
             
             primTableObjectId = atoi(idxNameComponents[2].c_str());
             indexName = idxNameComponents[3];
@@ -2387,7 +2591,7 @@ Ndb::getNdbErrorDetail(const NdbError& err, char* buff, Uint32 buffLen) const
             
             Uint32 components = tabName.split(tabNameComponents,
                                               splitString);
-            assert (components == 3);
+            require(components == 3);
             
             /* Now we generate a string of the format
              * <dbname>/<schemaname>/<tabname>/<idxname>
