@@ -26,7 +26,7 @@
 #include <NodeBitmask.hpp>
 #include <NdbEnv.h>
 
-extern my_bool opt_core;
+extern "C" my_bool opt_core;
 
 #define DBG(x) \
   do { g_info << x << " at line " << __LINE__ << endl; } while (0)
@@ -710,7 +710,7 @@ createFK(NdbDictionary::Dictionary * dict,
   {
     const NdbDictionary::Index * idx = dict->getIndex(childIdx->getName(),
                                                       pChild->getName());
-    assert(idx != 0);
+    require(idx != 0);
     childIdx = idx;
   }
 
@@ -986,25 +986,78 @@ runRSSsnapshotCheck(NDBT_Context* ctx, NDBT_Step* step)
   return NDBT_OK;
 }
 
-static
+/**
+ * BUG#19643174
+ * 
+ * Test cases that use TcResourceSnapshot and TcResourceCheckLeak have
+ * to be protected from race conditions. There are multiple variants of
+ * races to protect against.
+ * 
+ * 1) We wake up the user thread before we send the TC_COMMIT_ACK, this could
+ * lead to that we haven't released the commit ack markers before our
+ * DUMP_STATE_ORD arrives in the DBTC instances. To handle this we set
+ * TC_COMMIT_ACK to be sent immediate and even before the user thread is
+ * signalled.
+ * 
+ * 2) The sending of TC_COMMIT_ACK uses a method to send the signal without
+ * flushing for performance reasons. However in this case we need it to be
+ * sent immediate, this is also handled by the same flag as for 1).
+ *
+ * 3) The sending of DUMP_STATE_ORD can race the TC_COMMIT_ACK if we send it
+ * through the management server. To avoid this we send it directly to all
+ * nodes through a signal.
+ *
+ * 4) The TC_COMMIT_ACK can still be raced by the DUMP_STATE_ORD if they arrive
+ * in the same TCP/IP message. This is so since the data node receiver will
+ * not flush the signals to the threads until it has received all signals or
+ * some maximum value. When flushing it starts with low thread numbers, so the
+ * thread where CMVMI belongs (the main thread) will get its signal flushed
+ * before the TC threads gets their signals flushed. This means that a signal
+ * directly to TC can be raced by a signal to the same TC routed via the thread
+ * of the CMVMI. To avoid this we always route TC_COMMIT_ACK via CMVMI when
+ * the immediate flag has been set.
+ *
+ * The above 4 measures handles the TC_COMMIT_ACK resources. There is however
+ * also a number of resources kept until the complete phase is processed.
+ * There is no signal sent back to the API when the complete phase is
+ * completed, so there isn't much we can do in that respect. There is however
+ * a signal WAIT_GCP_REQ that can be sent that waits for the current global
+ * checkpoint to complete before sending WAIT_GCP_CONF, given that we have
+ * received a transaction with a certain GCP, we know that this signal will
+ * not return until the complete phase of our transactions are completed.
+ * It will actually wait also for the logs to be written and so forth, but
+ * this extra wait doesn't matter since it is simply delaying the test case
+ * somewhat. So by adding a call to forceGCPWait(1) we ensure that the
+ * complete phase is done before we proceed with checking for memory leaks.
+ */
+
+#include "../../src/ndbapi/ndb_internal.hpp"
+
 int
 runTransSnapshot(NDBT_Context* ctx, NDBT_Step* step)
 {
   NdbRestarter restarter;
+  Ndb *pNdb = GETNDB(step);
+
   g_info << "save all resource usage" << endl;
   int dump1[] = { DumpStateOrd::TcResourceSnapshot };
   restarter.dumpStateAllNodes(dump1, 1);
+  Ndb_internal::set_TC_COMMIT_ACK_immediate(pNdb, true);
   return NDBT_OK;
 }
 
-static
 int
 runTransSnapshotCheck(NDBT_Context* ctx, NDBT_Step* step)
 {
-  NdbRestarter restarter;
+  Ndb *pNdb = GETNDB(step);
+  NdbDictionary::Dictionary *pDict = pNdb->getDictionary();
   g_info << "save all resource usage" << endl;
-  int dump1[] = { DumpStateOrd::TcResourceCheckLeak };
-  restarter.dumpStateAllNodes(dump1, 1);
+  pDict->forceGCPWait(1);
+  Uint32 dump1[] = { DumpStateOrd::TcResourceCheckLeak };
+  if (Ndb_internal::send_dump_state_all(pNdb, dump1, 1) != 0)
+  {
+    return NDBT_FAILED;
+  }
   return NDBT_OK;
 }
 
@@ -1381,14 +1434,16 @@ runTransError(NDBT_Context* ctx, NDBT_Step* step)
   ctx->setProperty("LongSignalMemorySnapshot", Uint32(1));
   int mode = ctx->getProperty("TransMode", Uint32(0));
 
-  char errbuf[256];
   for (int i = 0; terrorCodes[i] != 0; i++)
   {
+#ifdef NDB_USE_GET_ENV
+    char errbuf[256];
     if (NdbEnv_GetEnv("NDB_ERR_CODE", errbuf, sizeof(errbuf)) != 0 &&
         atoi(errbuf) != terrorCodes[i])
     {
       continue;
     }
+#endif
     printf("testing errcode: %d\n", terrorCodes[i]);
     runLongSignalMemorySnapshotStart(ctx, step);
     runTransSnapshot(ctx, step);
