@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2003, 2010, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2003, 2015, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -67,7 +67,7 @@ int main(int argc, char** argv){
   
   Ndb_cluster_connection con(opt_ndb_connectstring, opt_ndb_nodeid);
   con.set_name("ndb_drop_index");
-  if(con.connect(12, 5, 1) != 0)
+  if(con.connect(opt_connect_retries, opt_connect_retry_delay, 1) != 0)
   {
     return NDBT_ProgramExit(NDBT_FAILED);
   }
